package com.brilliance.isc.transfer.transaction;

import com.brilliance.mda.runtime.mda.util.Systems;
import com.brilliance.isc.bo.lg.HitpamStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.HitpamStoreVo;
import com.brilliance.isc.vo.model.CbbVo;
import com.brilliance.isc.vo.model.Giacbs;
import org.mapstruct.*;
import org.mapstruct.factory.Mappers;


/**
 * Kitame Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring",uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
        )
public interface HitpamStructMapper extends IBaseStructMapper<HitpamStoreBo, HitpamStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    HitpamStructMapper INSTANCE = Mappers.getMapper(HitpamStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    HitpamStoreBo voTransferToBo(HitpamStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({
            @Mapping(target = "oldnidgrp",ignore = true),
            //修改交易，两个机构记账机构和所属机构忽略赋值
            @Mapping(target = "nidgrp.rec.branchinr",ignore = true),
            @Mapping(target = "nidgrp.rec.bchkeyinr",ignore = true)
    })
    void voUpdateToBo(HitpamStoreVo vo, @MappingTarget HitpamStoreBo bo);

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    HitpamStoreVo boTransferToVo(HitpamStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(HitpamStoreBo bo, @MappingTarget HitpamStoreVo vo);

    @AfterMapping
    default void afterUpdate(HitpamStoreVo vo, @MappingTarget HitpamStoreBo bo){
        Giacbs cbs = vo.getNidgrp().getCbs();
        if(cbs == null) return;
        CbbVo max = cbs.getMax();
        if(max == null) return;
        bo.getNidgrp().setAmtwrdtxtz(Systems.moneycnsmalltobig(String.valueOf(max.getAmt())));
        bo.getNidgrp().setFmtamtz(String.valueOf(max.getAmt()));
    }
}
